@@ -1,12 +1,12 @@
 package SVK::Command::Add;
 use strict;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command );
 use constant opt_recursive => 1;
 use SVK::XD;
 use SVK::I18N;
-use SVK::Util qw( $SEP is_symlink );
+use SVK::Util qw( $SEP is_symlink mimetype_is_text to_native);
 
 sub options {
     ('q|quiet'		=> 'quiet');
@@ -14,7 +14,7 @@ sub options {
 
 sub parse_arg {
     my ($self, @arg) = @_;
-    return $self->arg_condensed (@arg);
+    return $self->arg_condensed(@arg);
 }
 
 sub lock {
@@ -25,10 +25,10 @@ sub run {
     my ($self, $target) = @_;
 
     unless ($self->{recursive}) {
-	die loc ("%1 already under version control.\n", $target->{report})
-	    unless $target->{targets};
+	die loc ("%1 already under version control.\n", $target->report)
+	    unless $target->source->{targets};
 	# check for multi-level targets
-	for (@{$target->{targets}}) {
+	for (@{$target->source->{targets}}) {
 	    # XXX: consolidate sep for targets
 	    my ($parent) = m{^(.*)[/\Q$SEP\E]}o or next;
 	    die loc ("Please add the parent directory '%1' first.\n", $parent)
@@ -38,16 +38,17 @@ sub run {
     }
 
     $self->{xd}->checkout_delta
-	( %$target,
-	  xdroot => $target->root ($self->{xd}),
+	( $target->for_checkout_delta,
+	  xdroot => $target->create_xd_root,
 	  delete_verbose => 1,
 	  unknown_verbose => $self->{recursive},
 	  editor => SVK::Editor::Status->new
 	  ( notify => SVK::Notify->new
 	    ( cb_flush => sub {
 		  my ($path, $status) = @_;
-		  my ($copath, $report) = map { SVK::Target->copath ($_, $path) }
-		      @{$target}{qw/copath report/};
+	          to_native($path, 'path');
+		  my $copath = $target->copath($path);
+		  my $report = $target->report->subdir($path);
 
 		  $target->contains_copath ($copath) or return;
 		  die loc ("%1 already added.\n", $report)
@@ -55,27 +56,41 @@ sub run {
 
 		  return unless $status->[0] eq 'D';
 		  lstat ($copath);
-		  $self->do_add ('R', $copath, $report, !-d _)
+		  $self->_do_add ('R', $copath, $report, !-d _)
 		      if -e _;
 	      })),
 	  cb_unknown => sub {
-	      lstat ($_[1]);
-	      $self->do_add ('A', $_[1], SVK::Target->copath ($target->{report}, $_[0]),
-			     (!-d _ or is_symlink));
+	      my ($editor, $path) = @_;
+	      to_native($path, 'path');
+	      my $copath = $target->copath($path);
+	      my $report = $target->report->subdir($path);
+	      lstat ($copath);
+	      $self->_do_add ('A', $copath, $report, !-d _);
 	  },
 	);
+    return;
 }
 
 my %sch = (A => 'add', 'R' => 'replace');
 
-sub do_add {
+sub _do_add {
     my ($self, $st, $copath, $report, $autoprop) = @_;
+    my $newprop;
+    $newprop = $self->{xd}->auto_prop($copath) if $autoprop;
+
     $self->{xd}{checkout}->store ($copath,
 				  { '.schedule' => $sch{$st},
 				    $autoprop ?
-				    ('.newprop'  => $self->{xd}->auto_prop ($copath)) : ()});
-    print "$st   $report\n" unless $self->{quiet};
+				    ('.newprop'  => $newprop) : ()});
+    return if $self->{quiet};
 
+    # determine whether the path is binary
+    my $bin = q{};
+    if ( ref $newprop && $newprop->{'svn:mime-type'} ) {
+        $bin = ' - (bin)' if !mimetype_is_text( $newprop->{'svn:mime-type'} );
+    }
+
+    print "$st   $report$bin\n";
 }
 
 1;
