@@ -1,6 +1,6 @@
 package SVK::Command::Cleanup;
 use strict;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command );
 use SVK::I18N;
@@ -21,18 +21,18 @@ sub run {
     my ($self, @arg) = @_;
 
     if ($self->{all}) {
-        $self->{xd}{checkout}->store_recursively ('', {lock => undef});
+        $self->{xd}{checkout}->store ('', {lock => undef});
         print loc("Cleaned up all stalled locks.\n");
         return;
     }
 
     for (@arg) {
-	if ($self->{xd}{checkout}->get ($_->{copath})->{lock}) {
-	    print loc("Cleaned up stalled lock on %1.\n", $_->{copath});
-	    $self->{xd}{checkout}->store ($_->{copath}, {lock => undef});
+	if ($self->{xd}{checkout}->get ($_->copath_anchor)->{lock}) {
+	    print loc("Cleaned up stalled lock on %1.\n", $_->copath_anchor);
+	    $self->{xd}{checkout}->store ($_->copath_anchor, {lock => undef});
 	}
         else {
-	    print loc("Path %1 was not locked.\n", $_->{copath});
+	    print loc("Path %1 was not locked.\n", $_->copath_anchor);
 	}
     }
     return;
