@@ -1,10 +1,9 @@
 package SVK::Command::Describe;
 use strict;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command::Diff SVK::Command::Log);
 use SVK::XD;
-use SVK::Editor::Diff;
 
 sub options {
     ();
@@ -14,14 +13,19 @@ sub parse_arg {
     my ($self, @arg) = @_;
     return if $#arg < 0;
 
+    # Allow user to type "svk describe r12345", for easy copy-and-paste
+    # from "svk log".
+    $arg[0] =~ s/^r(\d+)$/$1/;
+
     return ($arg[0], $self->arg_depotroot($arg[1]));
 }
 
 sub run {
     my ($self, $chg, $target) = @_;
-    $self->{revspec} = $chg;
+    my $rev = $self->resolve_revision($target,$chg);
+    $self->{revspec} = [$rev];
     $self->SVK::Command::Log::run ($target);
-    $self->{revspec} = ($chg-1).":$chg";
+    $self->{revspec} = [$rev-1, $rev];
     $self->SVK::Command::Diff::run ($target);
 }
 
@@ -41,6 +45,8 @@ SVK::Command::Describe - Describe a change
 
 Displays the change in revision number I<REV> in the specified depot.
 It always shows the entire change even if you specified a particular target.
+(I<REV> can optionally have the prefix C<r>, just like the revisions reported
+from C<svk log>.)
 
 =head1 OPTIONS
 
