@@ -6,6 +6,8 @@ use base qw( SVK::Command::Commit );
 use SVK::I18N;
 use SVK::Util qw( HAS_SVN_MIRROR is_uri get_prompt traverse_history );
 
+use constant narg => undef;
+
 sub options {
     ('l|list'  => 'list',
      'd|delete|detach'=> 'detach',
@@ -27,10 +29,14 @@ sub parse_arg {
     my $path = shift(@arg);
 
     # Allow "svk mi uri://... //depot" to mean "svk mi //depot uri://"
-    if (is_uri($path)) {
+    if (is_uri($path) && $arg[0]) {
         ($arg[0], $path) = ($path, $arg[0]);
     }
 
+    if (defined (my $narg = $self->narg)) {
+	return unless $narg == (scalar @arg + 1);
+    }
+
     return ($self->arg_depotpath ($path), @arg);
 }
 
@@ -38,31 +44,23 @@ sub run {
     my ($self, $target, $source, @options) = @_;
     die loc ("%1 already exists.\n", $target->path)
 	if $target->root->check_path ($target->path);
-    $self->get_mirror_object ($target, $source, @options)->init
-	or die loc("%1 already mirrored, use 'svk mirror --detach' to remove it first.\n",
-		   $target->{depotpath});
+    $self->{xd}->mirror($target->repos)
+	->add_entry($target->path_anchor, $source, @options)
+	    or die loc("%1 already mirrored, use 'svk mirror --detach' to remove it first.\n",
+		   $target->depotpath);
     return;
 }
 
-sub get_mirror_object {
-    my ($self, $target, $source, @options) = @_;
-    return SVN::Mirror->new (target_path => $target->{path},
-			     source => $source,
-			     repospath => $target->{repospath},
-			     repos => $target->{repos},
-			     options => \@options,
-			     config => $self->{svnconfig},
-			     pool => SVN::Pool->new,
-			    );
-}
-
 package SVK::Command::Mirror::relocate;
 use base qw(SVK::Command::Mirror);
 use SVK::I18N;
 
 sub run {
     my ($self, $target, $source, @options) = @_;
-    $self->get_mirror_object ($target, $source, @options)->relocate;
+    $self->{xd}->mirror($target->repos)
+	->svnmirror_object($target->path_anchor,
+			   source => $source, options => \@options)
+	->relocate;
     return;
 }
 
@@ -70,16 +68,17 @@ package SVK::Command::Mirror::detach;
 use base qw(SVK::Command::Mirror);
 use SVK::I18N;
 
-sub run {
-    my ($self, $target, $source, @options) = @_;
-    my ($m, $mpath) = SVN::Mirror::is_mirrored ($target->{repos},
-						$target->{path});
+use constant narg => 1;
 
-    die loc("%1 is not a mirrored path.\n", $target->{depotpath}) if !$m;
-    die loc("%1 is inside a mirrored path.\n", $target->{depotpath}) if $mpath;
+sub run {
+    my ($self, $target) = @_;
+    my ($m, $mpath) = $target->is_mirrored;
+
+    die loc("%1 is not a mirrored path.\n", $target->depotpath) if !$m;
+    die loc("%1 is inside a mirrored path.\n", $target->depotpath) if $mpath;
 
     $m->delete(1); # remove svm:source and svm:uuid too
-    print loc("Mirror path '%1' detached.\n", $target->{depotpath});
+    print loc("Mirror path '%1' detached.\n", $target->depotpath);
     return;
 }
 
@@ -87,9 +86,11 @@ package SVK::Command::Mirror::upgrade;
 use base qw(SVK::Command::Mirror);
 use SVK::I18N;
 
+use constant narg => 1;
+
 sub run {
     my ($self, $target) = @_;
-    SVN::Mirror::upgrade ($target->{repos});
+    SVN::Mirror::upgrade ($target->repos);
     return;
 }
 
@@ -97,17 +98,12 @@ package SVK::Command::Mirror::unlock;
 use base qw(SVK::Command::Mirror);
 use SVK::I18N;
 
+use constant narg => 1;
+
 sub run {
     my ($self, $target) = @_;
-    my $m = SVN::Mirror->new
-	( target_path => $target->path,
-	  repos => $target->{repos},
-	  ignore_lock => 1,
-	  get_source => 1
-	);
-    $m->init;
-    $m->unlock ('force');
-    print loc ("mirror locks on %1 removed.\n", $target->{report});
+    $self->{xd}->mirror($target->repos)->unlock($target->path_anchor);
+    print loc ("mirror locks on %1 removed.\n", $target->report);
     return;
 }
 
@@ -123,7 +119,7 @@ sub parse_arg {
 }
 
 sub run {
-    my ($self, $target, $source, @options) = @_;
+    my ($self, $target) = @_;
     my $fmt = "%-20s\t%-s\n";
     printf $fmt, loc('Path'), loc('Source');
     print '=' x 60, "\n";
@@ -135,20 +131,11 @@ sub run {
 	    warn loc ("Depot /%1/ not loadable.\n", $depot);
 	    next;
 	}
-	my @paths = SVN::Mirror::list_mirror ($target->{repos});
-	my $fs = $target->{repos}->fs;
-	my $root = $fs->revision_root ($fs->youngest_rev);
-	my $name = $target->depotname;
-	foreach my $path (@paths) {
-	    eval {
-	    my $m = SVN::Mirror->new(
-                    target_path => $path,
-                    repos => $target->{repos},
-                    get_source => 1
-                );
-	    printf $fmt, "/$name$path", $m->{source};
-	    };
-	}
+	my %mirrors = $self->{xd}->mirror($target->repos)->entries;
+	foreach my $path (sort keys %mirrors) {
+	    my $m = $mirrors{$path};
+	    printf $fmt, '/'.$target->depotname.$path, $m->mirror->{source};
+	};
     }
     return;
 }
@@ -158,10 +145,14 @@ use base qw(SVK::Command::Mirror);
 use SVK::Util qw( traverse_history get_prompt );
 use SVK::I18N;
 
+use constant narg => 1;
+
 sub run {
     my ($self, $target, $source, @options) = @_;
-    $source = ("file://$target->{repospath}");
-    my $m = $self->get_mirror_object ($target, $source, @options);
+    $source = ("file://".$target->repospath);
+    my $m = $self->{xd}->mirror($target->repos)
+	->svnmirror_object($target->path_anchor,
+			   source => $source, options => \@options);
     $self->recover_headrev ($target, $m);
     $self->recover_list_entry ($target, $m);
     return;
@@ -216,9 +207,10 @@ sub recover_headrev {
         delete => { direct => 1, message => '' }
     )->run($target);
 
+    $target->refresh_revision;
     $self->command(
-        copy => { rev => $rev, direct  => 1, message => '' },
-    )->run($target => $target);
+        copy => { direct  => 1, message => '' },
+    )->run($target->new(revision => $rev) => $target->new);
 
     # XXX - race condition? should get the last committed rev instead
     $target->refresh_revision;
@@ -234,7 +226,7 @@ sub recover_headrev {
 sub recover_list_entry {
     my ($self, $target, $m) = @_;
 
-    my %mirrors = map { ($_ => 1) } SVN::Mirror::list_mirror ($target->{repos});
+    my %mirrors = map { ($_ => 1) } SVN::Mirror::list_mirror ($target->repos);
 
     return if $mirrors{$m->{target_path}}++;
 
@@ -243,7 +235,7 @@ sub recover_list_entry {
         $self->arg_depotpath ('/'.$target->depotname.'/'),
     );
 
-    print loc("%1 added back to the list of mirrored paths.\n", $target->{report}); 
+    print loc("%1 added back to the list of mirrored paths.\n", $target->report);
     return;
 }
 
@@ -276,7 +268,7 @@ SVK::Command::Mirror - Initialize a mirrored depotpath
 
  -l [--list]            : list mirrored paths
  -d [--detach]          : mark a depotpath as no longer mirrored
- --relocate             : relocate the mirror to another URI
+ --relocate             : change the upstream URI for the mirrored depotpath
  --recover              : recover the state of a mirror path
  --unlock               : forcibly remove stalled locks on a mirror
  --upgrade              : upgrade mirror state to the latest version
