@@ -1,6 +1,6 @@
 package SVK::Command::Resolved;
 use strict;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command );
 use constant opt_recursive => 0;
@@ -21,7 +21,7 @@ sub run {
     my ($self, @arg) = @_;
 
     for my $target (@arg) {
-	$self->{xd}->do_resolved ( %$target,
+	$self->{xd}->do_resolved ( $target->for_checkout_delta,
 				   recursive => $self->{recursive},
 				 );
     }
