@@ -9,7 +9,7 @@ use SVK::I18N;
 use SVK::Util qw( HAS_SVN_MIRROR );
 
 sub options {
-    ('r|revision=i'    => 'rev',
+    ('r|revision=s'    => 'rev',
      's|sync'          => 'sync',
      'm|merge'         => 'merge',
      'q|quiet'         => 'quiet',
@@ -32,20 +32,29 @@ sub lock {
 sub run {
     my ($self, @arg) = @_;
 
-    die loc ("--check-only cannot be used in conjunction with --sync or --merge.\n")
-        if defined $self->{check_only} && ($self->{merge} || $self->{sync});
+    die loc ("--check-only cannot be used in conjunction with --merge.\n")
+        if defined $self->{check_only} && $self->{merge};
 
     die loc ("--revision cannot be used in conjunction with --sync or --merge.\n")
 	if defined $self->{rev} && ($self->{merge} || $self->{sync});
 
     for my $target (@arg) {
-	my $update_target = SVK::Target->new
-	    ( %$target,
-	      path => $self->{update_target_path} || $target->{path},
-	      revision => defined $self->{rev} ?
-	      $self->{rev} : $target->{repos}->fs->youngest_rev,
-	      copath => undef
-	    );
+	my $update_target = $target->source->new;
+	$update_target->path($self->{update_target_path})
+	    if defined $self->{update_target_path};
+
+	my $rev = defined $self->{rev} ?
+	    $self->resolve_revision($target->new,$self->{rev}) :
+	    $target->repos->fs->youngest_rev;
+
+	if ($update_target->isa('SVK::Path::View')) {
+	    # always use the latest view layout
+	    $update_target->refresh_revision;
+	    $update_target->source->revision($rev);
+	}
+        else {
+	    $update_target->revision($rev);
+	}
 
         # Because merging under the copy anchor is unsafe, we always merge
         # to the most immediate copy anchor under copath root.
@@ -61,10 +70,7 @@ sub run {
 
             # Because syncing under the mirror anchor is impossible,
             # we always sync from the mirror anchor.
-            my ($m, $mpath) = SVN::Mirror::is_mirrored (
-                $sync_target->{repos},
-                $sync_target->{path}
-            );
+            my ($m, $mpath) = $sync_target->is_mirrored;
             $m->run if $m->{source};
         }
 
@@ -87,51 +93,74 @@ sub run {
 
 sub do_update {
     my ($self, $cotarget, $update_target) = @_;
-    my ($xdroot, $newroot) = map { $_->root ($self->{xd}) } ($cotarget, $update_target);
+    my $pool = SVN::Pool->new_default;
+    my $xdroot = $cotarget->create_xd_root;
+    my $newroot = $update_target->root;
     # unanchorified
-    my ($path, $copath) = @{$cotarget}{qw/path copath/};
-    my $report = $cotarget->{report};
-    my $kind = $newroot->check_path ($update_target->{path});
-    die loc("path %1 does not exist.\n", $update_target->{path})
-	if $kind == $SVN::Node::none;
+    my $report = $cotarget->report;
+    my $kind = $newroot->check_path ($update_target->path);
+    if ($kind == $SVN::Node::none) {
+	# if update target doesn't exist, only allows updating from
+	# something that exist.
+	die loc("Path %1 does not exist.\n", $update_target->depotpath)
+	    unless $xdroot->check_path($cotarget->path);
+
+	$cotarget->anchorify;
+	$update_target->anchorify;
+	# still in the checkout
+	if ($self->{xd}{checkout}->get($cotarget->copath)->{depotpath}) {
+	    $kind = $newroot->check_path($update_target->path_anchor);
+	}
+    }
+
+    my $content_revision = $update_target->isa('SVK::Path::View') ?
+	$update_target->source->revision : $update_target->revision;
+    print loc("Syncing %1(%2) in %3 to %4.\n", $cotarget->depotpath,
+	      $cotarget->path_anchor, $cotarget->copath,
+	      $content_revision);
 
-    print loc("Syncing %1(%2) in %3 to %4.\n", @{$cotarget}{qw( depotpath path copath )},
-	      $update_target->{revision});
     if ($kind == $SVN::Node::file ) {
 	$cotarget->anchorify;
 	$update_target->anchorify;
 	# can't use $cotarget->{path} directly since the (rev0, /) hack
-	($path, $copath) = @{$cotarget}{qw/path copath/};
-	$cotarget->{targets}[0] = $cotarget->{copath_target};
+	$cotarget->source->{targets}[0] = $cotarget->{copath_target};
     }
-    my $base = $cotarget;
-    $base = $base->new (path => '/')
+    my $base = $cotarget->as_depotpath;
+    $base = $base->new(path => '/')
 	if $xdroot->check_path ($base->path) == $SVN::Node::none;
-    unless (-e $cotarget->{copath}) {
+
+    unless (-e $cotarget->copath) {
 	die loc ("Checkout directory gone. Use 'checkout %1 %2' instead.\n",
-		 $update_target->{depotpath}, $cotarget->{report})
-	    unless $base->{path} eq '/';
-	mkdir ($cotarget->{copath}) or
-	    die loc ("Can't create directory %1 for checkout: %2.\n", $cotarget->{report}, $!);
+		 $update_target->depotpath, $cotarget->report)
+	    unless $base->path_anchor eq '/';
+	mkdir ($cotarget->copath) or
+	    die loc ("Can't create directory %1 for checkout: %2.\n", $cotarget->report, $!);
     }
 
     my $notify = SVK::Notify->new_with_report
-	($report, $cotarget->{targets}[0], 1);
+	($report, $cotarget->path_target, 1);
     $notify->{quiet}++ if $self->{quiet};
     my $merge = SVK::Merge->new
-	(repos => $cotarget->{repos}, base => $base, base_root => $xdroot,
+	(repos => $cotarget->repos, base => $base, base_root => $xdroot,
 	 no_recurse => !$self->{recursive}, notify => $notify, nodelay => 1,
 	 src => $update_target, dst => $cotarget, check_only => $self->{check_only},
+	 auto => 1, # not to print track-rename hint
 	 xd => $self->{xd});
-    $merge->run ($self->{xd}->get_editor (copath => $copath, path => $path,
-					  store_path => $update_target->path,
-					  check_only => $self->{check_only},
-					  ignore_checksum => 1,
-					  oldroot => $xdroot, newroot => $newroot,
-					  revision => $update_target->{revision},
-					  anchor => $cotarget->{path},
-					  target => $cotarget->{targets}[0] || '',
-					  update => $self->{check_only} ? 0 : 1));
+    my ($editor, $inspector, %cb) = $cotarget->get_editor
+	( ignore_checksum => 1,
+	  check_only => $self->{check_only},
+	  store_path => $update_target->path_anchor,
+	  update => $self->{check_only} ? 0 : 1,
+	  newroot => $newroot,
+	  revision => $content_revision,
+	);
+    $merge->run($editor, %cb, inspector => $inspector);
+
+    if ($update_target->isa('SVK::Path::View')) {
+	$self->{xd}{checkout}->store
+	    ($cotarget->copath,
+	     {depotpath => $update_target->depotpath});
+    }
 }
 
 1;
@@ -148,12 +177,12 @@ SVK::Command::Update - Bring changes from repository to checkout copies
 
 =head1 OPTIONS
 
- -r [--revision] REV	: act on revision REV instead of the head revision
+ -r [--revision] REV    : act on revision REV instead of the head revision
  -N [--non-recursive]   : do not descend recursively
  -C [--check-only]      : try operation but make no changes
  -s [--sync]            : synchronize mirrored sources before update
  -m [--merge]           : smerge from copied sources before update
- -q [--quiet]           : quiet mode
+ -q [--quiet]           : print as little as possible
 
 =head1 DESCRIPTION
 
