@@ -36,9 +36,9 @@ sub _verify {
 sub run {
     my ($self, $chg, $depot) = @_;
     my $target = $self->arg_depotpath ("/$depot/");
-    my $fs = $target->{repos}->fs;
+    my $fs = $target->repos->fs;
     my $sig = $fs->revision_prop ($chg, 'svk:signature');
-    return _verify ($target->{repos}, $sig, $chg)
+    return _verify ($target->repos, $sig, $chg)
 	if $sig;
     print "No signature found for change $chg at /$depot/.\n";
     return;
@@ -81,6 +81,8 @@ sub close_edit {
 	return;
     }
     # verify the content
+    my $header = '-----BEGIN PGP SIGNED MESSAGE-----';
+    $sig =~ s/^.*$header/$header/s;
     my ($anchor) = $sig =~ m/^ANCHOR: (.*)$/m;
     my ($path) = resolve_svm_source ($self->{repos}, split (':', $anchor));
     while ($sig =~ m/^MD5\s(.*?)\s(.*?)$/gm) {
