@@ -1,7 +1,9 @@
 package SVK::Editor::Delay;
 use strict;
-our $VERSION = $SVK::VERSION;
-our @ISA = qw(SVN::Delta::Editor);
+use SVK::Version;  our $VERSION = $SVK::VERSION;
+
+require SVN::Delta;
+use base 'SVK::Editor::ByPass';
 
 sub _open_pbaton {
     my ($self, $pbaton, $func) = @_;
