@@ -1,7 +1,9 @@
 package SVK::Editor::Patch;
 use strict;
-our $VERSION = $SVK::VERSION;
-our @ISA = qw(SVN::Delta::Editor);
+use SVK::Version;  our $VERSION = $SVK::VERSION;
+
+require SVN::Delta;
+use base 'SVK::Editor';
 
 =head1 NAME
 
@@ -72,6 +74,7 @@ sub emit {
 	if ($ret && $#$ret > 0) {
 	    my $stream = SVN::TxDelta::parse_svndiff (@$ret, 1, $pool);
 	    print $stream $svndiff;
+	    close $stream;
 	}
     }
     else {
