@@ -1,7 +1,9 @@
 package SVK::Editor::Sign;
-our @ISA = qw (SVN::Delta::Editor);
+
+require SVN::Delta;
+use base 'SVK::Editor::ByPass';
 use SVK::I18N;
-use SVK::Util qw (tmpfile);
+use autouse 'SVK::Util' => qw (tmpfile);
 
 sub add_file {
     my ($self, $path, @arg) = @_;
@@ -47,7 +49,7 @@ sub _sign_gpg {
     };
 
     open D, "$sigfile" or die loc("cannot open %1: %2", $sigfile, $!);
-    undef $/;
+    local $/;
     my $buf = <D>;
     unlink($sigfile);
     return $buf;
